/**
 ******************************************************************************
 *
 * @file        MG82F6P32_GPIO_DRV.C
 *
 * @brief       This is the header file of the SFR address.
 *
 * @par         Project
 *              MG82F6P32
 * @version     v1.00
 * @date        2024/03/12
 * @copyright   Copyright (c) 2019 MegaWin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS" without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose. The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */

#include "MG82F6P32_CONFIG.h"

/**
 *******************************************************************************
 * @brief       Get Port 6 Pin 0 Status
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = DRV_GPIO_GetPin60();
 * @endcode
 *******************************************************************************
 */
bool DRV_GPIO_GetPin60(void)
{
    _push_(SFRPI);
    __DRV_SFR_PageIndex(P6_Page);
    if (P6 & P6Pin0)
    {
        _pop_(SFRPI);
        return 1;
    }
    _pop_(SFRPI);
    return 0;
}

/**
 *******************************************************************************
 * @brief       Get Port 6 Pin 1 Status
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = DRV_GPIO_GetPin61();
 * @endcode
 *******************************************************************************
 */
bool DRV_GPIO_GetPin61(void)
{
    _push_(SFRPI);
    __DRV_SFR_PageIndex(P6_Page);
    if (P6 & P6Pin1)
    {
        _pop_(SFRPI);
        return 1;
    }
    _pop_(SFRPI);
    return 0;
}
