/**
 ******************************************************************************
 *
 * @file        MG82F6P32_GPIO_DRV.H
 *
 * @brief       This is the header file of the SFR address.
 *
 * @par         Project
 *              MG82F6P32
 * @version     v1.00
 * @date        2024/03/12
 * @copyright   Copyright (c) 2019 MegaWin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS" without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose. The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */

#ifndef __MG82F6P32_GPIO_DRV_H
#define __MG82F6P32_GPIO_DRV_H
/// @cond __DRV_GPIO_define

#define HIGH 0
#define LOW 1

#define FAST 1
#define NORMAL 0

/// @endcond

///@cond
bool DRV_GPIO_GetPin60(void);
bool DRV_GPIO_GetPin61(void);
///@endcond

//!@{
//! defgroup P0Pinx Pin number
#define P0PinALL (0xA6)
#define P0Pin1 (0x02)
#define P0Pin2 (0x04)
#define P0Pin5 (0x20)
#define P0Pin7 (0x80)
//!@}

//!@{
//! defgroup P0IOMODE Port0 IO mode
#define P0ALL_InputOnly (0xA6A600A6)
#define P01_InputOnly (0x02020002)
#define P02_InputOnly (0x04040004)
#define P05_InputOnly (0x20200020)
#define P07_InputOnly (0x80800080)

#define P0ALL_OpenDrainPullUp (0xA6A6A6A6)
#define P01_OpenDrainPullUp (0x02020202)
#define P02_OpenDrainPullUp (0x04040404)
#define P05_OpenDrainPullUp (0x20202020)
#define P07_OpenDrainPullUp (0x80808080)

#define P0ALL_OpenDrain (0xA6A60000)
#define P01_OpenDrain (0x02020000)
#define P02_OpenDrain (0x04040000)
#define P05_OpenDrain (0x20200000)
#define P07_OpenDrain (0x80800000)

#define P0ALL_PushPull (0xA6A6A600)
#define P01_PushPull (0x02020200)
#define P02_PushPull (0x04040400)
#define P05_PushPull (0x20202000)
#define P07_PushPull (0x80808000)
//!@}

/**
 *******************************************************************************
 * @brief       Initializing Port0 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port0 mode.
 *  @arg\b      P0ALL_InputOnly (Default)
 *  @arg\b      P01_InputOnly
 *  @arg\b      P02_InputOnly
 *  @arg\b      P05_InputOnly
 *  @arg\b      P07_InputOnly
 *  @arg\b      P0ALL_OpenDrainPullUp
 *  @arg\b      P01_OpenDrainPullUp
 *  @arg\b      P02_OpenDrainPullUp
 *  @arg\b      P05_OpenDrainPullUp
 *  @arg\b      P07_OpenDrainPullUp
 *  @arg\b      P0ALL_OpenDrain
 *  @arg\b      P01_OpenDrain
 *  @arg\b      P02_OpenDrain
 *  @arg\b      P05_OpenDrain
 *  @arg\b      P07_OpenDrain
 *  @arg\b      P0ALL_PushPull
 *  @arg\b      P01_PushPull
 *  @arg\b      P02_PushPull
 *  @arg\b      P05_PushPull
 *  @arg\b      P07_PushPull
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P0ModeSelect(P00_OpenDrainPullUp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P0ModeSelect(__MODE__)          \
    MWT(                                           \
        P0M0 = P0M0 & ~(HIBYTE(HIWORD(__MODE__))); \
        P0M0 = P0M0 | (HIBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(P0M1_Page);            \
        P0M1 = P0M1 & ~(LOBYTE(HIWORD(__MODE__))); \
        P0M1 = P0M1 | (LOBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port0 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port0 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP0M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP0M0(__MODE__) P0M0 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port0 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port0 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP0M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP0M0(__MODE__) __MODE__ = P0M0

/**
 *******************************************************************************
 * @brief       Set Port0 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port0 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP0M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP0M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P0M1_Page); \
        P0M1 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port0 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port0 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP0M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP0M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P0M1_Page); \
        __MODE__ = P0M1;                \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup P1Pinx Pin number
#define P1PinALL (0xFF)
#define P1Pin0 (0x01)
#define P1Pin1 (0x02)
#define P1Pin2 (0x04)
#define P1Pin3 (0x08)
#define P1Pin4 (0x10)
#define P1Pin5 (0x20)
#define P1Pin6 (0x40)
#define P1Pin7 (0x80)
//!@}

//!@{
//! defgroup P0IOMODE Port0 IO mode
#define P1ALL_InputOnly (0xFFFF00FF)
#define P10_InputOnly (0x01010001)
#define P11_InputOnly (0x02020002)
#define P12_InputOnly (0x04040004)
#define P13_InputOnly (0x08080008)
#define P14_InputOnly (0x10100010)
#define P15_InputOnly (0x20200020)
#define P16_InputOnly (0x40400040)
#define P17_InputOnly (0x80800080)

#define P1ALL_OpenDrainPullUp (0xFFFFFFFF)
#define P10_OpenDrainPullUp (0x01010101)
#define P11_OpenDrainPullUp (0x02020202)
#define P12_OpenDrainPullUp (0x04040404)
#define P13_OpenDrainPullUp (0x08080808)
#define P14_OpenDrainPullUp (0x10101010)
#define P15_OpenDrainPullUp (0x20202020)
#define P16_OpenDrainPullUp (0x40404040)
#define P17_OpenDrainPullUp (0x80808080)

#define P1ALL_OpenDrain (0xFFFF0000)
#define P10_OpenDrain (0x01010000)
#define P11_OpenDrain (0x02020000)
#define P12_OpenDrain (0x04040000)
#define P13_OpenDrain (0x08080000)
#define P14_OpenDrain (0x10100000)
#define P15_OpenDrain (0x20200000)
#define P16_OpenDrain (0x40400000)
#define P17_OpenDrain (0x80800000)

#define P1ALL_PushPull (0xFFFFFF00)
#define P10_PushPull (0x01010100)
#define P11_PushPull (0x02020200)
#define P12_PushPull (0x04040400)
#define P13_PushPull (0x08080800)
#define P14_PushPull (0x10101000)
#define P15_PushPull (0x20202000)
#define P16_PushPull (0x40404000)
#define P17_PushPull (0x80808000)
//!@}
/**
 *******************************************************************************
 * @brief       Initializing Port1 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port1 mode.
 *  @arg\b      P1ALL_InputOnly (Default)
 *  @arg\b      P10_InputOnly
 *  @arg\b      P11_InputOnly
 *  @arg\b      P12_InputOnly
 *  @arg\b      P13_InputOnly
 *  @arg\b      P14_InputOnly
 *  @arg\b      P15_InputOnly
 *  @arg\b      P16_InputOnly
 *  @arg\b      P17_InputOnly
 *  @arg\b      P1ALL_OpenDrainPullUp
 *  @arg\b      P10_OpenDrainPullUp
 *  @arg\b      P11_OpenDrainPullUp
 *  @arg\b      P12_OpenDrainPullUp
 *  @arg\b      P13_OpenDrainPullUp
 *  @arg\b      P14_OpenDrainPullUp
 *  @arg\b      P15_OpenDrainPullUp
 *  @arg\b      P16_OpenDrainPullUp
 *  @arg\b      P17_OpenDrainPullUp
 *  @arg\b      P1ALL_OpenDrain
 *  @arg\b      P10_OpenDrain
 *  @arg\b      P11_OpenDrain
 *  @arg\b      P12_OpenDrain
 *  @arg\b      P13_OpenDrain
 *  @arg\b      P14_OpenDrain
 *  @arg\b      P15_OpenDrain
 *  @arg\b      P16_OpenDrain
 *  @arg\b      P17_OpenDrain
 *  @arg\b      P1ALL_PushPull
 *  @arg\b      P10_PushPull
 *  @arg\b      P11_PushPull
 *  @arg\b      P12_PushPull
 *  @arg\b      P13_PushPull
 *  @arg\b      P14_PushPull
 *  @arg\b      P15_PushPull
 *  @arg\b      P16_PushPull
 *  @arg\b      P17_PushPull
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P1ModeSelect(P10_OpenDrainPullUp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P1ModeSelect(__MODE__)          \
    MWT(                                           \
        P1M0 = P1M0 & ~(HIBYTE(HIWORD(__MODE__))); \
        P1M0 = P1M0 | (HIBYTE(LOWORD(__MODE__)));  \
        P1M1 = P1M1 & ~(LOBYTE(HIWORD(__MODE__))); \
        P1M1 = P1M1 | (LOBYTE(LOWORD(__MODE__)));)

/**
 *******************************************************************************
 * @brief       Set Port1 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port1 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP1M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP1M0(__MODE__) P1M0 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port1 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port1 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP1M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP1M0(__MODE__) __MODE__ = P1M0

/**
 *******************************************************************************
 * @brief       Set Port1 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port1 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP1M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP1M1(__MODE__) P1M1 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port1 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port1 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP1M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP1M1(__MODE__) __MODE__ = P1M1

//!@{
//! defgroup P2Pinx Pin number
#define P2PinALL (0x5C)
#define P2Pin2 (0x04)
#define P2Pin3 (0x08)
#define P2Pin4 (0x10)
#define P2Pin6 (0x40)
//!@}

//!@{
//! defgroup P2IOMODE Port0 IO mode
#define P2ALL_InputOnly (0x5C5C005C)
#define P22_InputOnly (0x04040004)
#define P23_InputOnly (0x08080008)
#define P24_InputOnly (0x10100010)
#define P26_InputOnly (0x40400040)

#define P2ALL_OpenDrainPullUp (0x5C5C5C5C)
#define P22_OpenDrainPullUp (0x04040404)
#define P23_OpenDrainPullUp (0x08080808)
#define P24_OpenDrainPullUp (0x10101010)
#define P26_OpenDrainPullUp (0x40404040)

#define P2ALL_OpenDrain (0x5C5C0000)
#define P22_OpenDrain (0x04040000)
#define P23_OpenDrain (0x08080000)
#define P24_OpenDrain (0x10100000)
#define P26_OpenDrain (0x40400000)

#define P2ALL_PushPull (0x5C5C5C00)
#define P22_PushPull (0x04040400)
#define P23_PushPull (0x08080800)
#define P24_PushPull (0x10101000)
#define P26_PushPull (0x40404000)
//!@}

/**
 *******************************************************************************
 * @brief       Initializing Port2 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port2 mode.
 *  @arg\b      P2ALL_InputOnly (Default)
 *  @arg\b      P22_InputOnly
 *  @arg\b      P23_InputOnly
 *  @arg\b      P24_InputOnly
 *  @arg\b      P26_InputOnly
 *  @arg\b      P2ALL_OpenDrainPullUp
 *  @arg\b      P22_OpenDrainPullUp
 *  @arg\b      P23_OpenDrainPullUp
 *  @arg\b      P24_OpenDrainPullUp
 *  @arg\b      P26_OpenDrainPullUp
 *  @arg\b      P2ALL_OpenDrain
 *  @arg\b      P22_OpenDrain
 *  @arg\b      P23_OpenDrain
 *  @arg\b      P24_OpenDrain
 *  @arg\b      P26_OpenDrain
 *  @arg\b      P2ALL_PushPull
 *  @arg\b      P22_PushPull
 *  @arg\b      P23_PushPull
 *  @arg\b      P24_PushPull
 *  @arg\b      P26_PushPull
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P2ModeSelect(P20_OpenDrainPullUp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P2ModeSelect(__MODE__)          \
    MWT(                                           \
        P2M0 = P2M0 & ~(HIBYTE(HIWORD(__MODE__))); \
        P2M0 = P2M0 | (HIBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(P2M1_Page);            \
        P2M1 = P2M1 & ~(LOBYTE(HIWORD(__MODE__))); \
        P2M1 = P2M1 | (LOBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port2 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port2 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP2M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP2M0(__MODE__) P2M0 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port2 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port2 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP2M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP2M0(__MODE__) __MODE__ = P2M0

/**
 *******************************************************************************
 * @brief       Set Port2 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port2 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP2M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP2M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P2M1_Page); \
        P2M1 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port2 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port2 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP2M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP2M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P2M1_Page); \
        __MODE__ = P2M1;                \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup P3Pinx Pin number
#define P3PinALL (0x3F)
#define P3Pin0 (0x01)
#define P3Pin1 (0x02)
#define P3Pin2 (0x04)
#define P3Pin3 (0x08)
#define P3Pin4 (0x10)
#define P3Pin5 (0x20)
//!@}

//!@{
//! defgroup P3IOMODE Port0 IO mode
#define P3ALL_AnalogMode (0x0A3F)
#define P30_AnalogMode (0x0A01)
#define P31_AnalogMode (0x0A02)
#define P32_AnalogMode (0x0A04)
#define P33_AnalogMode (0x0A08)
#define P34_AnalogMode (0x0A10)
#define P35_AnalogMode (0x0A20)

#define P3ALL_QuasiMode (0x3F3F0000)
#define P30_QuasiMode (0x01010000)
#define P31_QuasiMode (0x02020000)
#define P32_QuasiMode (0x04040000)
#define P33_QuasiMode (0x08080000)
#define P34_QuasiMode (0x10100000)
#define P35_QuasiMode (0x20200000)

#define P3ALL_PushPull (0x3F3F003F)
#define P30_PushPull (0x01010001)
#define P31_PushPull (0x02020002)
#define P32_PushPull (0x04040004)
#define P33_PushPull (0x08080008)
#define P34_PushPull (0x10100010)
#define P35_PushPull (0x20200020)

#define P3ALL_InputOnly (0x3F3F3F00)
#define P30_InputOnly (0x01010100)
#define P31_InputOnly (0x02020200)
#define P32_InputOnly (0x04040400)
#define P33_InputOnly (0x08080800)
#define P34_InputOnly (0x10101000)
#define P35_InputOnly (0x20202000)

#define P3ALL_OpenDrain (0x3F3F3F3F)
#define P30_OpenDrain (0x01010101)
#define P31_OpenDrain (0x02020202)
#define P32_OpenDrain (0x04040404)
#define P33_OpenDrain (0x08080808)
#define P34_OpenDrain (0x10101010)
#define P35_OpenDrain (0x20202020)
//!@}

/**
 *******************************************************************************
 * @brief       Initializing Port3 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port3 mode.
 *  @arg\b      P3ALL_QuasiMode (Default)
 *  @arg\b      P30_QuasiMode
 *  @arg\b      P31_QuasiMode
 *  @arg\b      P32_QuasiMode
 *  @arg\b      P33_QuasiMode
 *  @arg\b      P34_QuasiMode
 *  @arg\b      P35_QuasiMode
 *  @arg\b      P3ALL_PushPull
 *  @arg\b      P30_PushPull
 *  @arg\b      P31_PushPull
 *  @arg\b      P32_PushPull
 *  @arg\b      P33_PushPull
 *  @arg\b      P34_PushPull
 *  @arg\b      P35_PushPull
 *  @arg\b      P3ALL_InputOnly
 *  @arg\b      P30_InputOnly
 *  @arg\b      P31_InputOnly
 *  @arg\b      P32_InputOnly
 *  @arg\b      P33_InputOnly
 *  @arg\b      P34_InputOnly
 *  @arg\b      P35_InputOnly
 *  @arg\b      P3ALL_OpenDrain
 *  @arg\b      P30_OpenDrain
 *  @arg\b      P31_OpenDrain
 *  @arg\b      P32_OpenDrain
 *  @arg\b      P33_OpenDrain
 *  @arg\b      P34_OpenDrain
 *  @arg\b      P35_OpenDrain
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P3ModeSelect(P30_QuasiMode);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P3ModeSelect(__MODE__)              \
    MWT(                                               \
        if ((HIBYTE(__MODE__)) == (0x0A)) {            \
            __DRV_SFR_PageIndex(P3M2_Page);            \
            P3M2 = P3M2 | (__MODE__ & 0x00FF);         \
            __DRV_SFR_PageIndex(0);                    \
            P3M0 = P3M0 & ~(__MODE__ & 0x00FF);        \
            P3M1 = P3M1 & ~(__MODE__ & 0x00FF);        \
        } else {                                       \
            __DRV_SFR_PageIndex(P3M2_Page);            \
            P3M2 = P3M2 & ~(HIBYTE(HIWORD(__MODE__))); \
            __DRV_SFR_PageIndex(0);                    \
            P3M0 = P3M0 & ~(HIBYTE(HIWORD(__MODE__))); \
            P3M0 = P3M0 | (HIBYTE(LOWORD(__MODE__)));  \
            P3M1 = P3M1 & ~(LOBYTE(HIWORD(__MODE__))); \
            P3M1 = P3M1 | (LOBYTE(LOWORD(__MODE__)));  \
        })

/**
 *******************************************************************************
 * @brief       Set Port3 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port3 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP3M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP3M0(__MODE__) P3M0 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port3 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port3 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP3M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP3M0(__MODE__) __MODE__ = P3M0

/**
 *******************************************************************************
 * @brief       Set Port3 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port3 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP3M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP3M1(__MODE__) P3M1 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port3 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port3 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP3M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP3M1(__MODE__) __MODE__ = P3M1

/**
 *******************************************************************************
 * @brief       Set Port3 Mode Register 2
 * @details
 * @param[in]   __MODE__ :  Set the Port3 mode register 2.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP3M2(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP3M2(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P3M2_Page); \
        P3M2 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port3 Mode Register 2
 * @details
 * @param[in]   __MODE__ :  Get the Port3 mode register 2.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP3M2(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP3M2(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P3M2_Page); \
        __MODE__ = P3M2;                \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup P4Pinx Pin number
#define P4PinALL (0xB3)
#define P4Pin0 (0x01)
#define P4Pin1 (0x02)
#define P4Pin4 (0x10)
#define P4Pin5 (0x20)
#define P4Pin7 (0x80)
//!@}

//!@{
//! defgroup P4IOMODE Port0 IO mode
#define P4ALL_InputOnly (0xB3B300B3)
#define P40_InputOnly (0x01010001)
#define P41_InputOnly (0x02020002)
#define P44_InputOnly (0x10100010)
#define P45_InputOnly (0x20200020)
#define P47_InputOnly (0x80800080)

#define P4ALL_OpenDrainPullUp (0xB3B3B3B3)
#define P40_OpenDrainPullUp (0x01010101)
#define P41_OpenDrainPullUp (0x02020202)
#define P44_OpenDrainPullUp (0x10101010)
#define P45_OpenDrainPullUp (0x20202020)
#define P47_OpenDrainPullUp (0x80808080)

#define P4ALL_OpenDrain (0xB3B30000)
#define P40_OpenDrain (0x01010000)
#define P41_OpenDrain (0x02020000)
#define P44_OpenDrain (0x10100000)
#define P45_OpenDrain (0x20200000)
#define P47_OpenDrain (0x80800000)

#define P4ALL_PushPull (0xB3B3B300)
#define P40_PushPull (0x01010100)
#define P41_PushPull (0x02020200)
#define P44_PushPull (0x10101000)
#define P45_PushPull (0x20202000)
#define P47_PushPull (0x80808000)
//!@}

/**
 *******************************************************************************
 * @brief       Initializing Port4 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port4 mode.
 *  @arg\b      P4ALL_InputOnly (Default)
 *  @arg\b      P40_InputOnly
 *  @arg\b      P41_InputOnly
 *  @arg\b      P44_InputOnly
 *  @arg\b      P45_InputOnly
 *  @arg\b      P47_InputOnly
 *  @arg\b      P4ALL_OpenDrainPullUp
 *  @arg\b      P40_OpenDrainPullUp
 *  @arg\b      P41_OpenDrainPullUp
 *  @arg\b      P44_OpenDrainPullUp
 *  @arg\b      P45_OpenDrainPullUp
 *  @arg\b      P47_OpenDrainPullUp
 *  @arg\b      P4ALL_OpenDrain
 *  @arg\b      P40_OpenDrain
 *  @arg\b      P41_OpenDrain
 *  @arg\b      P44_OpenDrain
 *  @arg\b      P45_OpenDrain
 *  @arg\b      P47_OpenDrain
 *  @arg\b      P4ALL_PushPull
 *  @arg\b      P40_PushPull
 *  @arg\b      P41_PushPull
 *  @arg\b      P44_PushPull
 *  @arg\b      P45_PushPull
 *  @arg\b      P47_PushPull
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P4ModeSelect(P40_InputOnly);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P4ModeSelect(__MODE__)          \
    MWT(                                           \
        P4M0 = P4M0 & ~(HIBYTE(HIWORD(__MODE__))); \
        P4M0 = P4M0 | (HIBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(P4M1_Page);            \
        P4M1 = P4M1 & ~(LOBYTE(HIWORD(__MODE__))); \
        P4M1 = P4M1 | (LOBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port4 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port4 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP4M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP4M0(__MODE__) P4M0 = __MODE__

/**
 *******************************************************************************
 * @brief       Get Port4 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port4 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP4M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP4M0(__MODE__) __MODE__ = P4M0

/**
 *******************************************************************************
 * @brief       Set Port4 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port4 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP4M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP4M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P4M1_Page); \
        P4M1 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port4 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port4 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP4M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP4M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P4M1_Page); \
        __MODE__ = P4M1;                \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup P6Pinx Pin number
#define P6PinALL (0x03)
#define P6Pin0 (0x01)
#define P6Pin1 (0x02)
//!@}

//!@{
//! defgroup P6IOMODE Port0 IO mode
#define P6ALL_InputOnly (0x03030003)
#define P60_InputOnly (0x01010001)
#define P61_InputOnly (0x02020002)

#define P6ALL_OpenDrainPullUp (0x03030303)
#define P60_OpenDrainPullUp (0x01010101)
#define P61_OpenDrainPullUp (0x02020202)

#define P6ALL_OpenDrain (0x03030000)
#define P60_OpenDrain (0x01010000)
#define P61_OpenDrain (0x02020000)

#define P6ALL_PushPull (0x03030300)
#define P60_PushPull (0x01010100)
#define P61_PushPull (0x02020200)
//!@}

/**
 *******************************************************************************
 * @brief       Default initial Port6 IO mode
 * @details
 * @param[in]   __MODE__ :  specifies the Port6 mode.
 *  @arg\b      P6ALL_InputOnly (Default)
 *  @arg\b      P60_InputOnly
 *  @arg\b      P61_InputOnly
 *  @arg\b      P6ALL_OpenDrainPullUp
 *  @arg\b      P60_OpenDrainPullUp
 *  @arg\b      P61_OpenDrainPullUp
 *  @arg\b      P6ALL_OpenDrain
 *  @arg\b      P60_OpenDrain
 *  @arg\b      P61_OpenDrain
 *  @arg\b      P6ALL_PushPull
 *  @arg\b      P60_PushPull
 *  @arg\b      P61_PushPull
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P6ModeSelect(P60_InputOnly);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P6ModeSelect(__MODE__)          \
    MWT(                                           \
        __DRV_SFR_PageIndex(P6M0_Page);            \
        P6M0 = P6M0 & ~(HIBYTE(HIWORD(__MODE__))); \
        P6M0 = P6M0 | (HIBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(P6M1_Page);            \
        P6M1 = P6M1 & ~(LOBYTE(HIWORD(__MODE__))); \
        P6M1 = P6M1 | (LOBYTE(LOWORD(__MODE__)));  \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port6 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Set the Port6 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP6M0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP6M0(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P6M0_Page); \
        P6M0 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port6 Mode Register 0
 * @details
 * @param[in]   __MODE__ :  Get the Port6 mode register 0.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP6M0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP6M0(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P6M0_Page); \
        __MODE__ = P6M0;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port6 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Set the Port6 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP6M1(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP6M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P6M1_Page); \
        P6M1 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port6 Mode Register 1
 * @details
 * @param[in]   __MODE__ :  Get the Port6 mode register 1.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP6M1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP6M1(__MODE__)    \
    MWT(                                \
        __DRV_SFR_PageIndex(P6M1_Page); \
        __MODE__ = P6M1;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Default initial Port0 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P0DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P0DeInit() \
    MWT(                      \
        __DRV_GPIO_P0ModeSelect(P0ALL_InputOnly);)

/**
 *******************************************************************************
 * @brief       Default initial Port1 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P1DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P1DeInit() \
    MWT(                      \
        __DRV_GPIO_P1ModeSelect(P1ALL_InputOnly);)

/**
 *******************************************************************************
 * @brief       Default initial Port2 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P2DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P2DeInit() \
    MWT(                      \
        __DRV_GPIO_P2ModeSelect(P2ALL_InputOnly);)

/**
 *******************************************************************************
 * @brief       Default initial Port3 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P3DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P3DeInit() \
    MWT(                      \
        __DRV_GPIO_P3ModeSelect(P3ALL_AnalogMode);)

/**
 *******************************************************************************
 * @brief       Default initial Port4 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P4DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P4DeInit() \
    MWT(                      \
        __DRV_GPIO_P4ModeSelect(P4ALL_InputOnly);)

/**
 *******************************************************************************
 * @brief       Default initial Port6 IO mode
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P6DeInit();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P6DeInit() \
    MWT(                      \
        __DRV_GPIO_P6ModeSelect(P6ALL_InputOnly);)

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port0 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP0(P00,1);
 *   __DRV_GPIO_WriteP0(P0,0x01);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP0(__PORT__, __VALUE__) __PORT__ = __VALUE__

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port1 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP1(P10,1);
 *   __DRV_GPIO_WriteP1(P1,0x01);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP1(__PORT__, __VALUE__) __PORT__ = __VALUE__

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port2 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP2(P20,1);
 *   __DRV_GPIO_WriteP2(P2,0x04);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP2(__PORT__, __VALUE__) __PORT__ = __VALUE__

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port3 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP3(P30,1);
 *   __DRV_GPIO_WriteP3(P3,0x01);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP3(__PORT__, __VALUE__) __PORT__ = __VALUE__

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port4 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP4(P45,1);
 *   __DRV_GPIO_WriteP4(P4,0x10);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP4(__PORT__, __VALUE__) __PORT__ = __VALUE__

/**
 *******************************************************************************
 * @brief       Writing GPIO's Port6 state.
 * @details
 * @param[in]   __PORT__ : specifies a whole port or port pins.
 * @param[in]   __VALUE__ : specifies the value to be written.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_WriteP6(P60,1);
 *   __DRV_GPIO_WriteP6(P6,0x01);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_WriteP6(__PORT__, __VALUE__) \
    MWT(                                        \
        __DRV_SFR_PageIndex(P6_Page);           \
        __PORT__ = __VALUE__;                   \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port0 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return      Port state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_ReadP0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP0(__DATA__) __DATA__ = P0

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port0 Pin1 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin01();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin01() P01

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port0 Pin2 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin02();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin02() P02

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port0 Pin5 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin05();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin05() P05

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port0 Pin7 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin07();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin07() P07

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return      Port state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_ReadP1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP1(__DATA__) __DATA__ = P1

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin0 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin10();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin10() P10

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin1 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin11();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin11() P11

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin2 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin12();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin12() P12

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin3 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin13();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin13() P13

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin4 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin14();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin14() P14

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin5 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin15();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin15() P15

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin6 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin16();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin16() P16

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port1 Pin7 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin17();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin17() P17

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port2 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return      Port state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_ReadP2(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP2(__DATA__) __DATA__ = P2

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port2 Pin2 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin22();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin22() P22

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port2 Pin3 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin23();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin23() P23

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port2 Pin4 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin24();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin24() P24

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port2 Pin6 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin26();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin26() P26

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return      Port state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_ReadP3(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP3(__DATA__) __DATA__ = P3

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin0 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin30();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin30() P30

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin1 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin31();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin31() P31

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin2 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin32();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin32() P32

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin3 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin33();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin33() P33

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin4 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin34();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin34() P34

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port3 Pin5 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin35();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin35() P35

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_Read(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP4(__DATA__) __DATA__ = P4

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 Pin0 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin40();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin40() P40

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 Pin1 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin41();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin41() P41

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 Pin4 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin44();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin44() P44

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 Pin5 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin45();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin45() P45

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port4 Pin7 state.
 * @details
 * @return      Pin state
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   tmp = __DRV_GPIO_GetPin47();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin47() P47

/**
 *******************************************************************************
 * @brief       Reading GPIO's Port6 state.
 * @details
 * @param[out]   __DATA__ : the place to store data.
 * @return
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_ReadP6(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_ReadP6(__DATA__)   \
    MWT(                              \
        __DRV_SFR_PageIndex(P6_Page); \
        __DATA__ = P6;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Inversing Port0 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P0
 *  @arg\b      P01
 *  @arg\b      P02
 *  @arg\b      P05
 *  @arg\b      P07
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P0_InversePinP0(P00);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP0(__PORT__) __PORT__ = ~(__PORT__)

/**
 *******************************************************************************
 * @brief       Inversing Port1 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P1
 *  @arg\b      P10
 *  @arg\b      P11
 *  @arg\b      P12
 *  @arg\b      P13
 *  @arg\b      P14
 *  @arg\b      P15
 *  @arg\b      P16
 *  @arg\b      P17
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P1_InversePinP1(P10);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP1(__PORT__) __PORT__ = ~(__PORT__)

/**
 *******************************************************************************
 * @brief       Inversing Port2 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P2
 *  @arg\b      P22
 *  @arg\b      P23
 *  @arg\b      P24
 *  @arg\b      P26
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P2_InversePinP2(P22);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP2(__PORT__) __PORT__ = ~(__PORT__)

/**
 *******************************************************************************
 * @brief       Inversing Port3 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P3
 *  @arg\b      P30
 *  @arg\b      P31
 *  @arg\b      P32
 *  @arg\b      P33
 *  @arg\b      P34
 *  @arg\b      P35
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P3_InversePinP3(P30);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP3(__PORT__) __PORT__ = ~(__PORT__)

/**
 *******************************************************************************
 * @brief       Inversing Port4 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P4
 *  @arg\b      P40
 *  @arg\b      P41
 *  @arg\b      P44
 *  @arg\b      P45
 *  @arg\b      P47
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P4_InversePinP4(P44);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP4(__PORT__) __PORT__ = ~(__PORT__)

/**
 *******************************************************************************
 * @brief       Inversing Port6 pin state.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P6
 *  @arg\b      P60
 *  @arg\b      P61
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_P6_InversePinP6(P60);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_InversePinP6(__PORT__) \
    MWT(                                  \
        __DRV_SFR_PageIndex(P6_Page);     \
        __PORT__ = ~(__PORT__);           \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup Pack_Port_Pins
#define P00_to_P03_Driving P0DC0
#define P04_to_P07_Driving P0DC1
//!@}

/**
 *******************************************************************************
 * @brief       Selecting Port0 driving current strength.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P00_to_P03_Driving
 *  @arg\b      P04_to_P07_Driving
 * @param[in]   __STATE__ :
 *  @arg\b      HIGH : specifies P0 with high driving current strength.
 *  @arg\b      LOW : specifies P0 with low driving current strength.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_OutputDrivingConfig(P00_to_P03_Driving, HIGH);
 *   __DRV_GPIO_OutputDrivingConfig(P04_to_P07_Driving, LOW);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P0OutputDrivingConfig(__PORT__, __STATE__)                                \
    MWT(                                                                                     \
        __DRV_SFR_PageIndex(PDRVC0_Page);                                                    \
        (__STATE__ == LOW) ? (PDRVC0 = PDRVC0 | __PORT__) : (PDRVC0 = PDRVC0 & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup Pack_Port_Pins
#define P10_to_P13_Driving P1DC0
#define P14_to_P17_Driving P1DC1
//!@}
/**
 *******************************************************************************
 * @brief       Selecting Port1 driving current strength.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P10_to_P13_Driving
 *  @arg\b      P14_to_P17_Driving
 * @param[in]   __STATE__ :
 *  @arg\b      HIGH : specifies P1 with high driving current strength.
 *  @arg\b      LOW : specifies P1 with low driving current strength.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_OutputDrivingConfig(P10_to_P13_Driving, HIGH);
 *   __DRV_GPIO_OutputDrivingConfig(P14_to_P17_Driving, LOW);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P1OutputDrivingConfig(__PORT__, __STATE__)                                \
    MWT(                                                                                     \
        __DRV_SFR_PageIndex(PDRVC0_Page);                                                    \
        (__STATE__ == LOW) ? (PDRVC0 = PDRVC0 | __PORT__) : (PDRVC0 = PDRVC0 & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup Pack_Port_Pins
#define P20_to_P23_Driving P2DC0
#define P24_to_P27_Driving P2DC1
//!@}
/**
 *******************************************************************************
 * @brief       Selecting Port2 driving current strength.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P20_to_P23_Driving
 *  @arg\b      P24_to_P27_Driving
 * @param[in]   __STATE__ :
 *  @arg\b      HIGH : specifies P2 with high driving current strength.
 *  @arg\b      LOW : specifies P2 with low driving current strength.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_OutputDrivingConfig(P20_to_P23_Driving, HIGH);
 *   __DRV_GPIO_OutputDrivingConfig(P24_to_P27_Driving, LOW);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P2OutputDrivingConfig(__PORT__, __STATE__)                                \
    MWT(                                                                                     \
        __DRV_SFR_PageIndex(PDRVC0_Page);                                                    \
        (__STATE__ == LOW) ? (PDRVC0 = PDRVC0 | __PORT__) : (PDRVC0 = PDRVC0 & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup Pack_Port_Pins
#define P30_to_P33_Driving P3DC0
#define P34_to_P37_Driving P3DC1
//!@}
/**
 *******************************************************************************
 * @brief       Selecting Port3 driving current strength.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P30_to_P33_Driving
 *  @arg\b      P34_to_P37_Driving
 * @param[in]   __STATE__ : enables high driving or low driving.
 *  @arg\b      HIGH : specifies P3 with high driving current strength.
 *  @arg\b      LOW : specifies P3 with low driving current strength.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_OutputDrivingConfig(P30_to_P33_Driving, HIGH);
 *   __DRV_GPIO_OutputDrivingConfig(P30_to_P33_Driving, LOW);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P3OutputDrivingConfig(__PORT__, __STATE__)                                \
    MWT(                                                                                     \
        __DRV_SFR_PageIndex(PDRVC0_Page);                                                    \
        (__STATE__ == LOW) ? (PDRVC0 = PDRVC0 | __PORT__) : (PDRVC0 = PDRVC0 & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port0/1/2/3 driving current strength register
 * @details
 * @param[in]   __MODE__ :  Set Port0/1/2/3 driving current strength register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetPDRVC0(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetPDRVC0(__MODE__)    \
    MWT(                                  \
        __DRV_SFR_PageIndex(PDRVC0_Page); \
        PDRVC0 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port0/1/2/3 driving current strength register
 * @details
 * @param[in]   __MODE__ :  Get Port0/1/2/3 driving current strength register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetPDRVC0(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPDRVC0(__MODE__)    \
    MWT(                                  \
        __DRV_SFR_PageIndex(PDRVC0_Page); \
        __MODE__ = PDRVC0;                \
        __DRV_SFR_PageIndex(0);)

//!@{
//! defgroup Pack_Port_Pins
#define P40_to_P43_Driving P4DC0
#define P44_to_P46_Driving P4DC1
//!@}
/**
 *******************************************************************************
 * @brief       Selecting Port4 driving current strength.
 * @details
 * @param[in]   __PORT__ : specifies port pins
 *  @arg\b      P40_to_P43_Driving
 *  @arg\b      P44_to_P46_Driving
 * @param[in]   __STATE__ : enables high driving or low driving.
 *  @arg\b      HIGH : specifies P4 with high driving current strength.
 *  @arg\b      LOW : specifies P4 with low driving current strength.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_OutputDrivingConfig(P44_to_P46_Driving, HIGH);
 *   __DRV_GPIO_OutputDrivingConfig(P44_to_P46_Driving, LOW);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P4OutputDrivingConfig(__PORT__, __STATE__)                                \
    MWT(                                                                                     \
        __DRV_SFR_PageIndex(PDRVC1_Page);                                                    \
        (__STATE__ == LOW) ? (PDRVC1 = PDRVC1 | __PORT__) : (PDRVC1 = PDRVC1 & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port4 driving current strength register
 * @details
 * @param[in]   __MODE__ :  Set Port4 driving current strength register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetPDRVC1(0x03);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetPDRVC1(__MODE__)    \
    MWT(                                  \
        __DRV_SFR_PageIndex(PDRVC1_Page); \
        PDRVC1 = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port4 driving current strength register
 * @details
 * @param[in]   __MODE__ :  Get Port4 driving current strength register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetPDRVC1(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPDRVC1(__MODE__)    \
    MWT(                                  \
        __DRV_SFR_PageIndex(PDRVC1_Page); \
        __MODE__ = PDRVC1;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Port0 pin output fast driving control.
 * @details
 * @param[in]   __PORT__ : port pin to enable fast driving.
 *  @arg\b      P0PinALL
 *  @arg\b      P0Pin1
 *  @arg\b      P0Pin2
 *  @arg\b      P0Pin5
 *  @arg\b      P0Pin7
 * @param[in]   __STATE__ : enables/disables fast driving.
 *  @arg\b      NORMAL : disables fast driving on port pin output. (Default)
 *  @arg\b      FAST : enables fast driving on port pin output.
 * @return  None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP0FastDriving(P0PinALL, NORMAL);
 *   __DRV_GPIO_SetP0FastDriving(P0PinALL, FAST);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP0FastDriving(__PORT__, __STATE__)                                  \
    MWT(                                                                                  \
        __DRV_SFR_PageIndex(P0FDC_Page);                                                  \
        __STATE__ == FAST ? (P0FDC = P0FDC | (__PORT__)) : (P0FDC = P0FDC & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port0 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Set Port0 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP0FDC(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP0FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P0FDC_Page); \
        P0FDC = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port0 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Get Port0 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP0FDC(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP0FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P0FDC_Page); \
        __MODE__ = P0FDC;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Port1 pin output fast driving control.
 * @details
 * @param[in]   __PORT__ : port pin to enable fast driving.
 *  @arg\b      P1PinALL
 *  @arg\b      P1Pin0
 *  @arg\b      P1Pin1
 *  @arg\b      P1Pin2
 *  @arg\b      P1Pin3
 *  @arg\b      P1Pin4
 *  @arg\b      P1Pin5
 *  @arg\b      P1Pin6
 *  @arg\b      P1Pin7
 * @param[in]   __STATE__ : enables/disables fast driving.
 *  @arg\b      NORMAL : disables fast driving on port pin output. (Default)
 *  @arg\b      FAST : enables fast driving on port pin output.
 * @return  None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP1FastDriving(P1PinALL, NORMAL);
 *   __DRV_GPIO_SetP1FastDriving(P1PinALL, FAST);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP1FastDriving(__PORT__, __STATE__)                                  \
    MWT(                                                                                  \
        __DRV_SFR_PageIndex(P1FDC_Page);                                                  \
        __STATE__ == FAST ? (P1FDC = P1FDC | (__PORT__)) : (P1FDC = P1FDC & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port1 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Set Port1 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP1FDC(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP1FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P1FDC_Page); \
        P1FDC = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port1 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Get Port1 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP1FDC(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP1FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P1FDC_Page); \
        __MODE__ = P1FDC;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Port2 pin output fast driving control.
 * @details
 * @param[in]   __PORT__ : port pin to enable fast driving.
 *  @arg\b      P2PinALL
 *  @arg\b      P2Pin2
 *  @arg\b      P2Pin3
 *  @arg\b      P2Pin4
 *  @arg\b      P2Pin6
 * @param[in]   __STATE__ : enables/disables fast driving.
 *  @arg\b      NORMAL : disables fast driving on port pin output. (Default)
 *  @arg\b      FAST : enables fast driving on port pin output.
 * @return  None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP2FastDriving(P2PinALL, NORMAL);
 *   __DRV_GPIO_SetP2FastDriving(P2PinALL, FAST);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP2FastDriving(__PORT__, __STATE__)                                  \
    MWT(                                                                                  \
        __DRV_SFR_PageIndex(P2FDC_Page);                                                  \
        __STATE__ == FAST ? (P2FDC = P2FDC | (__PORT__)) : (P2FDC = P2FDC & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port2 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Set Port2 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP2FDC(0xAC);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP2FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P2FDC_Page); \
        P2FDC = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port2 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Get Port2 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP2FDC(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP2FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P2FDC_Page); \
        __MODE__ = P2FDC;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Port3 pin output fast driving control.
 * @details
 * @param[in]   __PORT__ : port pin to enable fast driving.
 *  @arg\b      P3PinALL
 *  @arg\b      P3Pin0
 *  @arg\b      P3Pin1
 *  @arg\b      P3Pin2
 *  @arg\b      P3Pin3
 *  @arg\b      P3Pin4
 *  @arg\b      P3Pin5
 * @param[in]   __STATE__ : enables/disables fast driving.
 *  @arg\b      NORMAL : disables fast driving on port pin output. (Default)
 *  @arg\b      FAST : enables fast driving on port pin output.
 * @return  None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP3FastDriving(P3PinALL, NORMAL);
 *   __DRV_GPIO_SetP3FastDriving(P3PinALL, FAST);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP3FastDriving(__PORT__, __STATE__)                                  \
    MWT(                                                                                  \
        __DRV_SFR_PageIndex(P3FDC_Page);                                                  \
        __STATE__ == FAST ? (P3FDC = P3FDC | (__PORT__)) : (P3FDC = P3FDC & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port3 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Set Port3 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP3FDC(0x3F);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP3FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P3FDC_Page); \
        P3FDC = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port3 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Get Port3 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP3FDC(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP3FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P3FDC_Page); \
        __MODE__ = P3FDC;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Port4 pin output fast driving control.
 * @details
 * @param[in]   __PORT__ : port pin to enable fast driving.
 *  @arg\b      P4PinALL
 *  @arg\b      P4Pin0
 *  @arg\b      P4Pin1
 *  @arg\b      P4Pin4
 *  @arg\b      P4Pin5
 *  @arg\b      P4Pin7
 * @param[in]   __STATE__ : enables/disables fast driving.
 *  @arg\b      NORMAL : disables fast driving on port pin output. (Default)
 *  @arg\b      FAST : enables fast driving on port pin output.
 * @return  None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP4FastDriving(P4PinALL, NORMAL);
 *   __DRV_GPIO_SetP4FastDriving(P4PinALL, FAST);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP4FastDriving(__PORT__, __STATE__)                                  \
    MWT(                                                                                  \
        __DRV_SFR_PageIndex(P4FDC_Page);                                                  \
        __STATE__ == FAST ? (P4FDC = P4FDC | (__PORT__)) : (P4FDC = P4FDC & ~(__PORT__)); \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Set Port4 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Set Port4 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetP4FDC(0xB3);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetP4FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P4FDC_Page); \
        P4FDC = __MODE__;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       Get Port4 pin output fast driving control register.
 * @details
 * @param[in]   __MODE__ :  Get Port4 pin output fast driving control register.
 * @return      None
 * @note
 * @par Example
 * @code
 *   unsigned char tmp = 0;
 *   __DRV_GPIO_GetP4FDC(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetP4FDC(__MODE__)    \
    MWT(                                 \
        __DRV_SFR_PageIndex(P4FDC_Page); \
        __MODE__ = P4FDC;                \
        __DRV_SFR_PageIndex(0);)

/**
 *******************************************************************************
 * @brief       OCD enable.
 * @details
 * @param[in]   __STATE__ : enables/disables OCD interface on P44 and P45
 *  @arg\b      MW_ENABLE : enables OCD interface on P44 and P45. (Default)
 *  @arg\b      MW_DISABLE : disables OCD interface on P44 and P45.
 * @return None
 * @note
 * @par Example
 * @code
 *   __DRV_GPIO_SetOCD2IO(MW_ENABLE);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetOCD2IO(__STATE__)                                           \
    MWT(                                                                          \
        DRV_PageP_Read(DCON0_P);                                                  \
        __STATE__ == MW_DISABLE ? (IFD = IFD & ~(OCDE_P)) : (IFD = IFD | OCDE_P); \
        DRV_PageP_Write(DCON0_P, IFD);)

/**
 *******************************************************************************
 * @brief       Reset function on I/O.
 * @details
 * @param[in]   __STATE__ :
 *  @arg\b      MW_ENABLE : Selects I/O pad function for external reset input, RST. (Default)
 *  @arg\b      MW_DISABLE : Selects I/O pad function for P47.
 * @return      None
 * @note
 * @code
 *   __DRV_GPIO_SetRST2IO(MW_ENABLE);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_SetRST2IO(__STATE__)                                             \
    MWT(                                                                            \
        DRV_PageP_Read(DCON0_P);                                                    \
        __STATE__ == MW_DISABLE ? (IFD = IFD & ~(RSTIO_P)) : (IFD = IFD | RSTIO_P); \
        DRV_PageP_Write(DCON0_P, IFD);)

/**
 *******************************************************************************
 * @brief       P4 SFR access control.
 * @details
 * @param[in]   __STATE__ : enables/disables P4 SFR protection.
 *  @arg\b      MW_ENABLE : it will disable the P4 SFR modifications in page 0~F. P4 in page 0~F only keeps SFR read function.
 *                      But software always owns the modification capability in SFR Page P.
 *  @arg\b      MW_DISABLE : modifications and read functions are both available in page 0~F and Page P. (Default)
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P4_Protected( MW_ENABLE);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P4_Protected(__STATE__)                                         \
    MWT(                                                                           \
        DRV_PageP_Read(SPCON0_P);                                                  \
        __STATE__ == MW_ENABLE ? (IFD = IFD | P4CTL_P) : (IFD = IFD & ~(P4CTL_P)); \
        DRV_PageP_Write(SPCON0_P, IFD);)

/**
 *******************************************************************************
 * @brief       P6 SFR access control.
 * @details
 * @param[in]   __STATE__ : enables/disables P6 SFR protection.
 *  @arg\b      MW_ENABLE : it will disable the P6 SFR modifications in page 0~F. P6 in page 0~F only keeps SFR read function.
 *                      But software always owns the modification capability in SFR Page P.
 *  @arg\b      MW_DISABLE : modifications and read functions are both available in page 0~F and Page P. (Default)
 * @return      None
 * @note
 * @par Example
 * @code
 *  __DRV_GPIO_P6_Protected(MW_ENABLE);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_P6_Protected(__STATE__)                                         \
    MWT(                                                                           \
        DRV_PageP_Read(SPCON0_P);                                                  \
        __STATE__ == MW_ENABLE ? (IFD = IFD | P6CTL_P) : (IFD = IFD & ~(P6CTL_P)); \
        DRV_PageP_Write(SPCON0_P, IFD);)

/**
 *******************************************************************************
 * @brief       Get Port 6 Pin 0 Status
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *   bool tmp = 0;
 *   __DRV_GPIO_GetPin60(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin60(__VALUE__)                               \
    MWT(                                                             \
        __DRV_SFR_PageIndex(P6_Page);                                \
        (P6 & P6Pin0) == P6Pin0 ? (__VALUE__ = 1) : (__VALUE__ = 0); \
        __DRV_SFR_PageIndex(0x00);)

/**
 *******************************************************************************
 * @brief       Get Port 6 Pin 1 Status
 * @details
 * @return      None
 * @note
 * @par Example
 * @code
 *   bool tmp = 0;
 *   __DRV_GPIO_GetPin61(tmp);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPIO_GetPin61(__VALUE__)                               \
    MWT(                                                             \
        __DRV_SFR_PageIndex(P6_Page);                                \
        (P6 & P6Pin1) == P6Pin1 ? (__VALUE__ = 1) : (__VALUE__ = 0); \
        __DRV_SFR_PageIndex(0x00);)

/**
 *******************************************************************************
 * void function
 *******************************************************************************
 */
bool DRV_GPIO_GetPin60(void);
bool DRV_GPIO_GetPin61(void);


#endif
