/**
 ******************************************************************************
 *
 * @file        MG82F6P32_GPLCRC_DRV.H
 *
 * @brief       This is the C code format driver head file.
 *
 * @par         Project
 *              MG82F6P32
 * @version     V1.00
 * @date        2024/03/28
 * @copyright   Copyright (c) 2019 MegaWin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS" without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose. The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */


#ifndef _MG82F6P32_GPLCRC_DRV_H
#define _MG82F6P32_GPLCRC_DRV_H

/**
 *******************************************************************************
 * @brief       Select GPLCRC Mode
 * @details
 * @param[in]   __SELECT__ : Set GPLCRC Mode to 16bit or 32bit
 *  @arg\b      GPLCRC_MODE_16BIT: Select the DMA mode to CRC16
 *  @arg\b      GPLCRC_MODE_32BIT: Select the DMA mode to CRC32
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SelectMODE(GPLCRC_MODE_32BIT);
 * @endcode
 *******************************************************************************
 */
#define GPLCRC_MODE_16BIT       0x00
#define GPLCRC_MODE_32BIT       CRCM0
#define __DRV_GPLCRC_SelectMODE(__SELECT__)\
        MWT(\
        __DRV_SFR_PageIndex(AUXR12_Page);\
        AUXR12 = (AUXR12 & ~CRCM0) | __SELECT__;\
        __DRV_SFR_PageIndex(0);\
    )
		
/**
 *******************************************************************************
 * @brief       Select CRC0DA is CRC0DI
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPGCRC_SelectCRC0DI();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPGCRC_SelectCRC0DI()        AUXR1 |= (CRCDS1 | CRCDS0)


/**
 *******************************************************************************
 * @brief       Select CRC0DA is CRC0SL
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPGCRC_SelectCRC0SL();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPGCRC_SelectCRC0SL()        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0)


/**
 *******************************************************************************
 * @brief       Select CRC0DA is CRC0SH
 * @details
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPGCRC_SelectCRC0SH();
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPGCRC_SelectCRC0SH()\
        MWT(\
           AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
           AUXR1 |= CRCDS0;\
        )


/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0SI data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0SI (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0SI(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0SI(__DATA__)\
        MWT(\
				AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= (CRCDS1 | CRCDS0);\
        CRC0DA = (__DATA__);\
    )


/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0SL data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0SL (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0SL(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0SL(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        CRC0DA = (__DATA__);\
    )


/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0SH data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0SH (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0SH(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0SH(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= CRCDS0;\
        CRC0DA = (__DATA__);\
    )


/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0S2 data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0S2 (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0S2(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0S2(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= CRCDS2;\
        CRC0DA = (__DATA__);\
    )
		
		
/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0S3 data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0S3 (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0S3(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0S3(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= (CRCDS2 | CRCDS0);\
        CRC0DA = (__DATA__);\
    )



/**
 *******************************************************************************
 * @brief       Set GPLCRC CRC0DA data
 * @details
 * @param[in]   __DATA__ : Set GPLCRC data to CRC0DA (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_SetCRC0DA(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_SetCRC0DA(__DATA__)       CRC0DA = (__DATA__)



/**
 *******************************************************************************
 * @brief       Get GPLCRC CRC0SL data
 * @details
 * @param[in]   __DATA__ : Get GPLCRC CRC0SL data (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_GetCRC0SL(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_GetCRC0SL(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        (__DATA__) = CRC0DA;\
    )


/**
 *******************************************************************************
 * @brief       Get GPLCRC CRC0SH data
 * @details
 * @param[in]   __DATA__ : Get GPLCRC CRC0SH data (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_GetCRC0SH(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_GetCRC0SH(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= CRCDS0;\
        (__DATA__) = CRC0DA;\
    )


/**
 *******************************************************************************
 * @brief       Get GPLCRC CRC0S2 data
 * @details
 * @param[in]   __DATA__ : Get GPLCRC CRC0S2 data (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_GetCRC0S2(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_GetCRC0S2(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= CRCDS1;\
        (__DATA__) = CRC0DA;\
    )
		
/**
 *******************************************************************************
 * @brief       Get GPLCRC CRC0S3 data
 * @details
 * @param[in]   __DATA__ : Get GPLCRC CRC0S3 data (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_GetCRC0S3(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_GetCRC0S3(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= (CRCDS1 | CRCDS0);\
        (__DATA__) = CRC0DA;\
    )


/**
 *******************************************************************************
 * @brief       Get GPLCRC CRC0DA data
 * @details
 * @param[in]   __DATA__ : Get GPLCRC CRC0DA data (0x00 ~ 0xFF)
 * @return      None
 * @note
 * @par         Example
 * @code
 *    __DRV_GPLCRC_GetCRC0DA(0xFF);
 * @endcode
 *******************************************************************************
 */
#define __DRV_GPLCRC_GetCRC0DA(__DATA__)\
        MWT(\
        AUXR1 &= ~(CRCDS2 | CRCDS1 | CRCDS0);\
        AUXR1 |= CRCDS0;\
        (__DATA__) = CRC0DA;\
    )


void DRV_CRC16_Seed(uint8_t CRC0SH_t,uint8_t CRC0SL_t);
void DRV_CRC32_Seed(uint8_t CRC0S3_t,uint8_t CRC0S2_t,uint8_t CRC0SH_t,uint8_t CRC0SL_t);

#endif  //_MG82F6P32_GPLCRC_DRV_H
