/**
 ******************************************************************************
 *
 * @file        MG82F6P32_OPAx_DRV.h
 *
 * @brief       This is the C code format driver head file.
 *
 * @par         Project
 *              MG82F6P32
 * @version     v1.00
 * @date        2024/03/26
 * @copyright   Copyright (c) 2019 Megawin Technology Co., Ltd.
 *              All rights reserved.
 *
 ******************************************************************************
 * @par         Disclaimer
 *      The Demo software is provided "AS IS" without any warranty, either
 *      expressed or implied, including, but not limited to, the implied warranties
 *      of merchantability and fitness for a particular purpose. The author will
 *      not be liable for any special, incidental, consequential or indirect
 *      damages due to loss of data or any other reason.
 *      These statements agree with the world wide and local dictated laws about
 *      authorship and violence against these laws.
 ******************************************************************************
 ******************************************************************************
 */
#ifndef _MG82F6P32_OPAx_DRV_H
#define _MG82F6P32_OPAx_DRV_H
/**
 *******************************************************************************
 * @brief     Select OPA0 OPA1 PowerMode
 * @details   Set OPPWR OP0LP OP1LP
 * @param[in] __SELECT__: config OPA0 OPA1  PowerMode
 *  @arg\b    OPA0 :High Power           OPA1 :High Power        (OPPWR=0,OP0LP=0,OP1LP=0)(Default)
 *  @arg\b    OPA0 :High Power           OPA1 :Medium High Power (OPPWR=0,OP0LP=0,OP1LP=1)
 *  @arg\b    OPA0 :Medium High Power    OPA1 :High Power        (OPPWR=0,OP0LP=1,OP1LP=0) 
 *  @arg\b    OPA0 :Medium High Power    OPA1 :Medium High Power (OPPWR=0,OP0LP=1,OP1LP=1)
 *  @arg\b    OPA0 :Medium Low Power     OPA1 :Medium Low Power  (OPPWR=1,OP0LP=0,OP1LP=0)
 *  @arg\b    OPA0 :Medium Low Power     OPA1 :Low Power         (OPPWR=1,OP0LP=0,OP1LP=1)
 *  @arg\b    OPA0 :Low Power            OPA1 :Medium Low Power  (OPPWR=1,OP0LP=1,OP1LP=0)
 *  @arg\b    OPA0 :Low Power            OPA1 :Low Power         (OPPWR=1,OP0LP=1,OP1LP=1)
 * @return    none
 * @note
 * @par       Example
 * @code
 *    __DRV_OPAx_PowerMode_Select(OPA_OPA0HP_OPA1HP);
 * @endcode
 *******************************************************************************
 */
/// @cond __DRV_OPAx_PowerMode_Select
#define OPA_OPA0HP_OPA1HP   0x01
#define OPA_OPA0HP_OPA1MHP  0x02
#define OPA_OPA0MHP_OPA1HP  0x03
#define OPA_OPA0MHP_OPA1MHP 0x04
#define OPA_OPA0MLP_OPA1MLP 0x05
#define OPA_OPA0MLP_OPA1LP  0x06
#define OPA_OPA0LP_OPA1MLP  0x07
#define OPA_OPA0LP_OPA1LP   0x08
/// @endcond
#define __DRV_OPAx_PowerMode_Select(__SELECT__)\
    MWT(\
        __DRV_SFR_PageIndex(OP0CON0_Page);\
        OP0CON0 = OP0CON0 & ~(OP0LP | OPPWR);\
        (__SELECT__^1)==0?(OP0CON0=(OP0CON0&~(OP0LP|OPPWR))):\
        (__SELECT__^2)==0?(OP0CON0=(OP0CON0&~(OP0LP|OPPWR))):\
        (__SELECT__^3)==0?(OP0CON0=(OP0CON0|(OP0LP))):\
        (__SELECT__^4)==0?(OP0CON0=(OP0CON0|(OP0LP))):\
        (__SELECT__^5)==0?(OP0CON0=(OP0CON0|(OPPWR))):\
        (__SELECT__^6)==0?(OP0CON0=(OP0CON0|(OPPWR))):\
        (__SELECT__^7)==0?(OP0CON0=(OP0CON0|(OP0LP|OPPWR))):\
        (__SELECT__^8)==0?(OP0CON0=(OP0CON0|(OP0LP|OPPWR))):_nop_();\
        __DRV_SFR_PageIndex(OP1CON0_Page);\
        OP1CON0 = OP1CON0 & ~(OP1LP);\
        (__SELECT__^1)==0?(OP1CON0=(OP1CON0&~(OP1LP))):\
        (__SELECT__^2)==0?(OP1CON0=(OP1CON0|(OP1LP))):\
        (__SELECT__^3)==0?(OP1CON0=(OP1CON0&~(OP1LP))):\
        (__SELECT__^4)==0?(OP1CON0=(OP1CON0|(OP1LP))):\
        (__SELECT__^5)==0?(OP1CON0=(OP1CON0&~(OP1LP))):\
        (__SELECT__^6)==0?(OP1CON0=(OP1CON0|(OP1LP))):\
        (__SELECT__^7)==0?(OP1CON0=(OP1CON0&~(OP1LP))):\
        (__SELECT__^8)==0?(OP1CON0=(OP1CON0|(OP1LP))):_nop_();\
        __DRV_SFR_PageIndex(0);\
    )
#endif

