/*********************************************************************
*    Project: MG82F6P32-DEMO
*			MG82F6P32 LQFP32_V10 EV Board
*			CpuCLK=12MHz, SysCLK=12MHz
*    Description:
*			CH0(P22),CH1(P23),CH2(P24),CH3(P34),CH4(P32),CH5(P35)
*    Note:
* 
*    Creat time:2024.02.26
*    Modify:
*    
*********************************************************************/
#define _MAIN_C

#include <Intrins.h>
#include <Absacc.h>

#include <Stdio.h>  // for printf

#include ".\include\REG_MG82F6P32.H"
#include ".\include\Type.h"
#include ".\include\API_Macro_MG82F6P32.H"
#include ".\include\API_Uart_BRGRL_MG82F6P32.H"

/*************************************************
*Set SysClk (MAX.50MHz)
*Selection: 
*	11059200,12000000,
*	22118400,24000000,
*	29491200,32000000,
*	44236800,48000000
*************************************************/
#define MCU_SYSCLK		32000000
/*************************************************/
/*************************************************
*Set CpuClk (MAX.36MHz)
*	1) CpuCLK=SysCLK
*	2) CpuClk=SysClk/2
*************************************************/
#define MCU_CPUCLK		(MCU_SYSCLK)
//#define MCU_CPUCLK		(MCU_SYSCLK/2)

#define TIMER_1T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) /256) 			
#define TIMER_1T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) %256)

#define TIMER_12T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) /256) 			
#define TIMER_12T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) %256)
#define TIMER_12T_10mS_TH	((65536-(u16)(float)(10000*((float)(MCU_SYSCLK)/12/(float)(1000000)))) /256) 			
#define TIMER_12T_10ms_TL	((65536-(u16)(float)(10000*((float)(MCU_SYSCLK)/12/(float)(1000000)))) %256)

#define IO_LED_G_0		P05
#define IO_LED_R		P26
#define IO_LED_G_1		P07


#define PTM0_RELOAD			(1000)
#define PTM0_CH0_SET_POINT	(100)
#define PTM0_CH0_CLR_POINT	(300)
#define PTM0_CH2_SET_POINT	(400)
#define PTM0_CH2_CLR_POINT	(600)
#define PTM0_CH4_SET_POINT	(700)
#define PTM0_CH4_CLR_POINT	(900)

#define PTM0_C           (65536)       	

#define PTM0_CL(x)		(u8)((~(u16)x)+1) 
#define PTM0_CH(x)     	(u8)(((~(u16)x)+1)>>8)        


/***********************************************************************************
*Function:   void INT_PCA(void)
*Description:PTM0 interrupt handle
		 
*Input:   
*Output:     
*************************************************************************************/
void INT_PTM0(void) interrupt INT_VECTOR_PTM0
{
	_push_(SFRPI);
	SFRPI=0;
	if(CF)
	{
		IO_LED_R=!IO_LED_R;
		CF=0;
		// Todo...
	}
	_pop_(SFRPI);
}


/*************************************************
*Function:     	void DelayXus(u16 xUs)
*Description:   	delay,unit:us
*Input:     		u8 Us -> *1us  (1~255)
*Output:     
*************************************************/
void DelayXus(u8 xUs)
{
	while(xUs!=0)
	{
#if (MCU_CPUCLK>=11059200)
		_nop_();
#endif
#if (MCU_CPUCLK>=14745600)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=16000000)
		_nop_();
#endif

#if (MCU_CPUCLK>=22118400)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=24000000)
		_nop_();
		_nop_();
#endif		
#if (MCU_CPUCLK>=29491200)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=32000000)
		_nop_();
		_nop_();
#endif

		xUs--;
	}
}

/*************************************************
*Function:     	void DelayXms(u16 xMs)
*Description:    delay,unit:ms
*Input:     		u16 xMs -> *1ms  (1~65535)
*Output:     
*************************************************/
void DelayXms(u16 xMs)
{
	while(xMs!=0)
	{
		CLRWDT();
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		xMs--;
		
	}
}

/***********************************************************************************
*Function:   	void InitPort()
*Description:	Initialize IO Port
*Input:   
*Output:   		
*************************************************************************************/
void InitPort(void)
{
	PORT_SetP0PushPull(BIT5|BIT7);					// Set P05,P07 as push-pull(LED)
	PORT_SetP2PushPull(BIT6);						// Set P26 as push-pull(LED)
	PORT_SetP2PushPull(BIT2|BIT3|BIT4);				// Set P22(CEX0),P23(CEX2),P24(CEX4) as push-pull
	PORT_SetP3PushPull(BIT2|BIT4|BIT5);				// Set P32(CEX1),P34(CEX3),P35(CEX5) as push-pull
}

/***********************************************************************************
*Function:   void InitPTM0_PhaseVariable(void)
*Description:   Initialize PTM0
*Input:   
*Output:     
*************************************************************************************/
void InitPTM0_PhaseVariable(void)
{
	PTM0_SetCLOCK_SYSCLK();								// PTM0 SysClk = 32MHz
	
	PTM0_CH0_SetMode_COPM();							// Set Compare output mode
	PTM0_CH1_SetMode_COPM();							// Set Compare output mode
	PTM0_CH2_SetMode_COPM();							// Set Compare output mode
	PTM0_CH3_SetMode_COPM();							// Set Compare output mode
	PTM0_CH4_SetMode_COPM();							// Set Compare output mode
	PTM0_CH5_SetMode_COPM();							// Set Compare output mode
	
	PTM0_CH0_SetPWM_16Bit();							// Set 16bit
	PTM0_CH1_SetPWM_16Bit();							// Set 16bit
	PTM0_CH2_SetPWM_16Bit();							// Set 16bit
	PTM0_CH3_SetPWM_16Bit();							// Set 16bit
	PTM0_CH4_SetPWM_16Bit();							// Set 16bit
	PTM0_CH5_SetPWM_16Bit();							// Set 16bit
	
	PTM0_CH01_SetPWM_EnBufferMode();					// CH0,CH1 Enable buffer mode
	PTM0_CH23_SetPWM_EnBufferMode();					// CH2,CH3 Enable buffer mode
	PTM0_CH45_SetPWM_EnBufferMode();					// CH4,CH5 Enable buffer mode
	
	PTM0_SetCounter(PTM0_C-PTM0_RELOAD);
	PTM0_SetCounterReload(PTM0_C-PTM0_RELOAD);

	PTM0_CH0_SetValue(PTM0_CH(PTM0_CH0_SET_POINT),PTM0_CL(PTM0_CH0_SET_POINT));		// CEX0 Set point
	PTM0_CH1_SetValue(PTM0_CH(PTM0_CH0_CLR_POINT),PTM0_CL(PTM0_CH0_CLR_POINT));		// CEX0 Clear point
	PTM0_CH2_SetValue(PTM0_CH(PTM0_CH2_SET_POINT),PTM0_CL(PTM0_CH2_SET_POINT));		// CEX2 Set point
	PTM0_CH3_SetValue(PTM0_CH(PTM0_CH2_CLR_POINT),PTM0_CL(PTM0_CH2_CLR_POINT));		// CEX2 Clear point
	PTM0_CH4_SetValue(PTM0_CH(PTM0_CH4_SET_POINT),PTM0_CL(PTM0_CH4_SET_POINT));		// CEX4 Set point
	PTM0_CH5_SetValue(PTM0_CH(PTM0_CH4_CLR_POINT),PTM0_CL(PTM0_CH4_CLR_POINT));		// CEX4 Clear point

	// Set PWM output pin
	PTM0_SetCEXn_Pin(CEX0_P22|CEX1_P23|CEX2_P24|CEX3_P34|CEX4_P32|CEX5_P35);	

	PTM0_SetPWM0_EnOutput();					
	PTM0_SetPWM1_EnOutput();
	PTM0_SetPWM2_EnOutput();
	PTM0_SetPWM3_EnOutput();
	PTM0_SetPWM4_EnOutput();
	PTM0_SetPWM5_EnOutput();
		
	PTM0_CF_EnInterrupt();					// Enable CF interrupt
	
	PTM0_EnPCACounter();					// Eanble PTM0 counter
}

/***********************************************************************************
*Function:   	void InitClock()
*Description:	Initialize clock
*Input:   
*Output:   		
*************************************************************************************/
void InitClock(void)
{
#if (MCU_SYSCLK==11059200)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=11.0592MHz CpuClk=11.0592MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1);
	
#else
	// SysClk=11.0592MHz CpuClk=5.5296MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1);
#endif
#endif

#if (MCU_SYSCLK==12000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=12MHz CpuClk=12MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1);
	
#else
	// SysClk=12MHz CpuClk=6MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1);
#endif
#endif

#if (MCU_SYSCLK==22118400)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=22.1184MHz CpuClk=22.1184MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#else
	// SysClk=22.1184MHz CpuClk=11.0592MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==24000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=24MHz CpuClk=24MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#else
	// SysClk=24MHz CpuClk=12MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==29491200)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// Cpuclk high speed
	CLK_SetCpuCLK_HighSpeed();
	// SysClk=29.491200MHz CpuClk=29.491200MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#else
	// SysClk=29.491200MHz CpuClk=14.7456MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==32000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// Cpuclk high speed
	CLK_SetCpuCLK_HighSpeed();
	// SysClk=32MHz CpuClk=32MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#else
	// SysClk=32MHz CpuClk=16MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==44236800)
	// SysClk=44.2368MHz CpuClk=22.1184MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx8, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X8|OSCIn_IHRCO);
#endif

#if (MCU_SYSCLK==48000000)
	// SysClk=48MHz CpuClk=24MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx8, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X8|OSCIn_IHRCO);
#endif

	// P60 output MCK/4
	//CLK_P60OC_MCKDiv4();
}

/***********************************************************************************
*Function:   	void InitInterrupt()
*Description:	Initialize Interrupt
*Input:   
*Output:   		
*************************************************************************************/
void InitInterrupt(void)
{
	INT_EnPTM0();					// Enable PTM0 interrupt
	
}	

/***********************************************************************************
*Function:       void InitSystem(void)
*Description:    Initialize MCU
*Input:   
*Output:     
*************************************************************************************/
void InitSystem(void)
{
	InitClock();
	InitPort();
	InitPTM0_PhaseVariable();
	InitInterrupt();

	INT_EnAll();
}

void main()
{

    InitSystem();
	
	IO_LED_G_0=0;IO_LED_R=0;IO_LED_G_1=0;
    DelayXms(1000);
	IO_LED_G_0=1;IO_LED_R=1;IO_LED_G_1=1;

	while(1)
    {
    	DelayXms(200);
		IO_LED_G_0=!IO_LED_G_0;
    }
}

